var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
import { eitherArrayReduce, eitherMapRight, Left, optionFold, optionFromNullable, Right, } from 'trimop';
import { RefField } from './data';
export function FilterSyncedFieldsError(p) {
    return __assign({}, p);
}
/**
 * Filter synced fields.
 * @param doc to be filtered
 * @param syncedFields
 * @returns synced fields if exists, undefined otherwise.
 */
export function filterSyncedFields(_a) {
    var doc = _a.doc, syncedFields = _a.syncedFields;
    return eitherArrayReduce(Object.entries(syncedFields), Right(undefined), function (acc, _a) {
        var fieldName = _a[0], syncFieldSpec = _a[1];
        return optionFold(optionFromNullable(doc[fieldName]), function () { return Right(acc); }, function (field) {
            var _a;
            // Copy the field if defined in the spec
            if (syncFieldSpec === true) {
                return Right(__assign(__assign({}, acc), (_a = {}, _a[fieldName] = field, _a)));
            }
            if (field._type !== 'Ref') {
                return Left(FilterSyncedFieldsError({
                    expectedFieldTypes: ['Ref'],
                    field: field,
                }));
            }
            // Copy nested synced fields
            return eitherMapRight(filterSyncedFields({
                doc: field.snapshot.doc,
                syncedFields: syncFieldSpec,
            }), function (syncedDoc) {
                return optionFold(optionFromNullable(syncedDoc), 
                // If there was no copied field
                function () { return Right(acc); }, function (syncedDoc) {
                    var _a;
                    return Right(__assign(__assign({}, acc), (_a = {}, _a[fieldName] = RefField({
                        doc: syncedDoc,
                        id: field.snapshot.id,
                    }), _a)));
                });
            });
        });
    });
}
//# sourceMappingURL=filter-synced-fields.js.map